<?php

require_once "AbstractManufacturer.php";

/**
 * Class OperationManufacturer
 *
 * Date: 25.03.13
 * Time: 15:12
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class OperationManufacturer extends AbstractManufacturer{

	/**
	 * A Singleton Instance of the OperationManufacturer
	 *
	 * @see OperatoinManufacturer::_getInstance()
	 * @see OperationManufacturer::__construct()
	 *
	 * @var null|OperationManufacturer
	 */
	protected static $instance = null;

	/**
	 * Get an Instance of OperationManufacturer
	 *
	 * @return OperationManufacturer
	 */
	public static function _getInstance(){
		if(self::$instance == null){
			self::$instance = new OperationManufacturer();
		}
		return self::$instance;
	}

	/**
	 * Create a new specific Composite object which is representing the delivered Attribute
	 *
	 * @param string $attribute
	 *
	 * @return AbstractComposite|AndOperation|EqualsOperation|GreaterOperation|GreaterOrEqualsOperation|LesserOperation|LesserOrEqualsOperation|NotEqualsOperation|OrOperation
	 * @throws UnsupportedOperation
	 */
	public function manufacture($attribute)
	{
		$operation = "";
		switch($attribute)
		{
			case LesserOperation::$pattern:
				$operation = new LesserOperation();
				break;
			case LesserOrEqualsOperation::$pattern:
				$operation = new LesserOrEqualsOperation();
				break;
			case EqualsOperation::$pattern:
				$operation = new EqualsOperation();
				break;
			case GreaterOrEqualsOperation::$pattern:
				$operation = new GreaterOrEqualsOperation();
				break;
			case GreaterOperation::$pattern:
				$operation = new GreaterOperation();
				break;
			case NotEqualsOperation::$pattern:
				$operation = new NotEqualsOperation();
				break;
			case AndOperation::$pattern:
				$operation = new AndOperation();
				break;
			case OrOperation::$pattern:
				$operation = new OrOperation();
				break;
			default:
				throw new UnsupportedOperation($attribute);
				break;
		}
		return $operation;
	}

	/**
	 * This function create a regular expression to match all operators in a condition. <br />
	 * The following string is created by this function <b>'/[\!&\|<>=]+/'</b><br />
	 * It matches all operators in a condition and is divided into the following parts:
	 *
	 * <pre>
	 * NEGATION:           !
	 * AND:                &
	 * OR:                 |
	 * LESSER:             <
	 * LESSER OR EQUALS:   <=
	 * EQUALS              =
	 * GREATER OR EQUALS   >=
	 * GREATER             >
	 * NOT EQUALS          <>
	 * </pre>
	 *
	 * @return string
	 */
	public function getPattern()
	{
//		return '/[&\|<>=]+/';
		return '/&|\||(?<!<|>)=|<(?!=|>)|>(?!=)|<=|>=|<>/';
	}

	/**
	 * Private construtor to prevent creating an object of OperationManufacturer
	 */
	private function __construct()
	{
		require_once __DIR__ . '/../Operations/LesserOperation.php';
		require_once __DIR__ . '/../Operations/LesserOrEqualsOperation.php';
		require_once __DIR__ . '/../Operations/EqualsOperation.php';
		require_once __DIR__ . '/../Operations/GreaterOrEqualsOperation.php';
		require_once __DIR__ . '/../Operations/GreaterOperation.php';
		require_once __DIR__ . '/../Operations/NotEqualsOperation.php';
		require_once __DIR__ . '/../Operations/AndOperation.php';
		require_once __DIR__ . '/../Operations/OrOperation.php';
		require_once __DIR__ . '/../Exception/UnsupportedOperation.php';
	}

	/**
	 * Private clone to prevent cloning an object of OperationManufacturer
	 */
	private function __clone() {}
}
