<?php

require_once "abstract.ilPluginModel.php";
require_once "./Modules/TestQuestionPool/classes/class.assQuestion.php";

/**
 * Class ilQuestionSetPoolNode
 *
 * Date: 21.11.13
 * Time: 11:42
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolNode extends ilPluginModel
{

	/**
	 * @var integer
	 */
	protected $id;

	/**
	 * @var integer
	 */
	protected $index;

	/**
	 * @var assQuestion
	 */
	protected $question;

	/**
	 * @var int
	 */
	protected $question_index;

	/**
	 * @var string|null
	 */
	protected $initial_condition;

	/**
	 * @var ilQuestionSetPoolPath
	 */
	protected $path;

	/**
	 * @var ilQuestionSetPoolJumpConditionList
	 */
	protected $jump_condition_list;

	/**
	 * @var int
	 */
	protected $dominant_scoring = NULL;

	/**
	 * @var string null
	 */
	protected $true_feedback = NULL;
	
	/**
	 * @var string null
	 */
	protected $false_feedback = NULL;

	public function __construct($id = 0)
	{
		require_once "class.ilQuestionSetPoolJumpConditionList.php";
		$this->jump_condition_list = new ilQuestionSetPoolJumpConditionList($this);

		parent::__construct($id);
	}

	/**
	 * @param int $id
	 */
	public function setId($id)
	{
		$this->id = $id;
	}

	/**
	 * @return int
	 */
	public function getId()
	{
		return $this->id;
	}

	/**
	 * @param int $index
	 */
	public function setIndex($index)
	{
		$this->index = $index;
	}

	/**
	 * @return int
	 */
	public function getIndex()
	{
		return $this->index;
	}

	/**
	 * @param null|string $initial_condition
	 */
	public function setInitialCondition($initial_condition)
	{
		$this->initial_condition = $initial_condition;
	}

	/**
	 * @return null|string
	 */
	public function getInitialCondition()
	{
		return $this->initial_condition;
	}

	/**
	 * @param \assQuestion $question
	 */
	public function setQuestion($question)
	{
		$this->question = $question;
	}

	/**
	 * @return \assQuestion
	 */
	public function getQuestion()
	{
		return $this->question;
	}

	/**
	 * @param \ilQuestionSetPoolPath $path
	 */
	public function setPath(ilQuestionSetPoolPath $path)
	{
		$this->path = $path;
	}

	/**
	 * @return \ilQuestionSetPoolPath
	 */
	public function getPath()
	{
		return $this->path;
	}

	/**
	 * @param \ilQuestionSetPoolJumpConditionList $jump_condition_list
	 */
	public function setJumpConditionList($jump_condition_list)
	{
		$this->jump_condition_list = $jump_condition_list;
	}

	/**
	 * @return \ilQuestionSetPoolJumpConditionList
	 */
	public function getJumpConditionList()
	{
		return $this->jump_condition_list;
	}

	/**
	 * @return int
	 */
	public function getQuestionIndex()
	{
		return $this->question_index;
	}

	/**
	 * @param int $dominant_scoring
	 */
	public function setDominantScoring($dominant_scoring)
	{
		$this->dominant_scoring = $dominant_scoring;
	}

	/**
	 * @return int
	 */
	public function getDominantScoring()
	{
		return $this->dominant_scoring;
	}

	/**
	 * @param integer $id
	 * @return bool
	 */
	public static function exists($id)
	{
		/**
		 * @var $ilDB ilDB
		 */
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT COUNT(*) cnt FROM rep_robj_xqsp_node WHERE id = %s",
			array("integer"),
			array($id)
		);
		$data = $ilDB->fetchAssoc($result);

		return (int)$data['cnt'] == 1 ? true : false;
	}

	public function update()
	{
		/**
		 * @var ilDB $ilDB
		 */
		global $ilDB;

		if($this->id > 0)
		{
			return (bool)$ilDB->update(
				"rep_robj_xqsp_node",
				array(
					"question_fi" 		=> array('integer', $this->getQuestion()->getId()),
					"initial_condition" => array('text', $this->getInitialCondition()),
					"dominant_scoring" 	=> array("integer", $this->getDominantScoring()),
					"true_feedback"		=> array('clob', $this->getTrueFeedback()),
					"false_feedback"	=> array('clob', $this->getFalseFeedback()),
				),
				array(
					'id' => array("integer", $this->getId())
				)
			);
		}
		else
		{
			$this->setId($ilDB->nextId("rep_robj_xqsp_node"));
			
			return $ilDB->queryF(
				 "INSERT INTO rep_robj_xqsp_node (id, path_fi, question_fi, node_index)
				(SELECT %s,%s, %s,
				CASE WHEN MAX(node_index) IS NULL
					THEN 1
					ELSE MAX(node_index) + 1
				END
				FROM rep_robj_xqsp_node WHERE path_fi = %s)",
				 array("integer", "integer", "integer","integer"),
				 array($this->getId(), $this->getPath()->getId(), $this->getQuestion()->getId(), $this->getPath()->getId())
			);
		}
	}

	public function read()
	{
		require_once "class.ilQuestionSetPoolPath.php";

		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT node.*, qst.question_index FROM rep_robj_xqsp_node node
			 INNER JOIN rep_robj_xqsp_qs_qst qst ON qst.question_fi = node.question_fi
			 WHERE id = %s",
				array("integer"),
				array($this->id)
		);

		$data = $ilDB->fetchAssoc($result);

		$this->setId($data['id']);
		$this->setIndex($data['node_index']);
		$this->setInitialCondition($data['initial_condition']);
		$this->setDominantScoring($data["dominant_scoring"]);
		$this->setQuestion(assQuestion::_instantiateQuestion($data['question_fi']));
		$this->question_index = $data["question_index"];
		
		require_once 'Services/RTE/classes/class.ilRTE.php';
		$this->setTrueFeedback(ilRTE::_replaceMediaObjectImageSrc($data['true_feedback'], 1));
		$this->setFalseFeedback(ilRTE::_replaceMediaObjectImageSrc($data['false_feedback'], 1));

		$path_fi = (isset($_GET['path_id']))? $_GET['path_id'] : $data['path_fi'];
		$path = new ilQuestionSetPoolPath($path_fi);
		$path->read();
		$this->setPath($path);
	}

	/**
	 * @param ilQuestionSetPoolPath $path
	 * @param array $question_ids
	 */
	public function duplicate($path, $question_ids)
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		$clone = clone $this;
		$clone->setId($ilDB->nextId("rep_robj_xqsp_node"));
		$clone->setPath($path);

		$ilDB->insert(
			"rep_robj_xqsp_node",
			array(
				"id"                => array("integer", $clone->getId()),
				"path_fi"           => array("integer", $clone->getPath()->getId()),
				"question_fi"       => array("integer", $question_ids[$this->getQuestion()->getId()]),
//				"node_index"        => array("integer", $clone->getIndex()),
				"node_index"        => array("integer", $this->getQuestionIndex()),
				"initial_condition" => array("text", $clone->getInitialCondition()),
				"dominant_scoring"  => array("integer", $this->getDominantScoring()),
				"true_feedback"		=> array("clob", $clone->getTrueFeedback()),
				"false_feedback"	=> array("clob", $clone->getFalseFeedback()),
			)
		);

		require_once "class.ilQuestionSetPoolJumpCondition.php";
		$data = $this->getJumpConditionList()->getData();
		foreach($data as $element)
		{
			$condition = new ilQuestionSetPoolJumpCondition($element["id"]);
			$condition->read();
			$condition->duplicate($clone, $question_ids);
		}

		return true;
	}

	public function remove()
	{
		global $ilDB;

		$this->removeJumpConditions();

		return $ilDB->manipulateF(
			"DELETE FROM rep_robj_xqsp_node WHERE id = %s",
			array("integer"),
			array($this->getId())
		);
	}

	/**
	 * @param ilPropertyFormGUI $form
	 */
	public function bindForm(ilPropertyFormGUI $form)
	{
		require_once 'Services/RTE/classes/class.ilRTE.php';

		if(($input = $form->getInput("initial_condition")) !== null)
		{
			$this->setInitialCondition($input);
		}

		if(($input = $form->getInput("question_fi")) != null)
		{
			$this->setQuestion(assQuestion::_instanciateQuestion($input));
		}
		
		if(($input = $form->getInput("true_feedback"))!= NULL)
		{
			$this->setTrueFeedback(ilRTE::_replaceMediaObjectImageSrc($input, 0));
		}
		
		if(($input = $form->getInput("false_feedback"))!= NULL)
		{
			$this->setFalseFeedback(ilRTE::_replaceMediaObjectImageSrc($input, 0));
		}
	}

	private function removeJumpConditions()
	{
		require_once "class.ilQuestionSetPoolJumpCondition.php";
		$data = $this->getJumpConditionList()->getData();
		foreach ($data as $element)
		{
			$condition = new ilQuestionSetPoolJumpCondition($element["id"]);
			$condition->read();
			$condition->remove();
		}
	}

	/**
	 * @return string
	 */
	public function getFalseFeedback()
	{
		return $this->false_feedback;
	}

	/**
	 * @param string $false_feedback
	 */
	public function setFalseFeedback($false_feedback)
	{
		$this->false_feedback = $false_feedback;
	}

	/**
	 * @return string
	 */
	public function getTrueFeedback()
	{
		return $this->true_feedback;
	}

	/**
	 * @param string $true_feedback
	 */
	public function setTrueFeedback($true_feedback)
	{
		$this->true_feedback = $true_feedback;
	}
	
	public function import()
	{
		global $ilDB;
		
		$this->setId($ilDB->nextId("rep_robj_xqsp_node"));
		
		$ilDB->queryF(
				"INSERT INTO rep_robj_xqsp_node (id, path_fi, question_fi, node_index)
				(SELECT %s,%s, %s,
				CASE WHEN MAX(node_index) IS NULL
					THEN 1
					ELSE MAX(node_index) + 1
				END
				FROM rep_robj_xqsp_node WHERE path_fi = %s)",
				array("integer", "integer", "integer","integer"),
				array($this->getId(), $this->getPath()->getId(), $this->getQuestion()->getId(), $this->getPath()->getId())
		);
		
		$ilDB->update(
				"rep_robj_xqsp_node",
				array(
						"initial_condition" => array('text', $this->getInitialCondition()),
						"dominant_scoring" 	=> array("integer", $this->getDominantScoring()),
						"true_feedback"		=> array('clob', $this->getTrueFeedback()),
						"false_feedback"	=> array('clob', $this->getFalseFeedback())
				),
				array('id' => array("integer", $this->getId())
				)
		);
	}
}
 